@@ -1,7 +1,8 @@
 import testenv; testenv.configure_for_tests()
-import sys, weakref
-from sqlalchemy import create_engine, exceptions, select, MetaData, Table, Column, Integer, String
-from testlib import *
+import weakref
+from testlib.sa import select, MetaData, Table, Column, Integer, String, pool
+import testlib.sa as tsa
+from testlib import TestBase, testing, engines
 import time
 import gc
 
@@ -35,7 +36,7 @@ class MockConnection(object):
 class MockCursor(object):
     def __init__(self, parent):
         self.explode = parent.explode
-        self.description = None
+        self.description = ()
     def execute(self, *args, **kwargs):
         if self.explode[0]:
             raise MockDisconnect("Lost the DB connection")
@@ -44,13 +45,14 @@ class MockCursor(object):
     def close(self):
         pass
 
+db, dbapi = None, None
 class MockReconnectTest(TestBase):
     def setUp(self):
         global db, dbapi
         dbapi = MockDBAPI()
 
         # create engine using our current dburi
-        db = create_engine('postgres://foo:bar@localhost/test', module=dbapi)
+        db = tsa.create_engine('postgres://foo:bar@localhost/test', module=dbapi)
 
         # monkeypatch disconnect checker
         db.dialect.is_disconnect = lambda e: isinstance(e, MockDisconnect)
@@ -81,7 +83,7 @@ class MockReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError:
+        except tsa.exc.DBAPIError:
             pass
 
         # assert was invalidated
@@ -110,7 +112,7 @@ class MockReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError:
+        except tsa.exc.DBAPIError:
             pass
 
         # assert was invalidated
@@ -123,7 +125,7 @@ class MockReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.InvalidRequestError, e:
+        except tsa.exc.InvalidRequestError, e:
             assert str(e) == "Can't reconnect until invalid transaction is rolled back"
 
         assert trans.is_active
@@ -131,7 +133,7 @@ class MockReconnectTest(TestBase):
         try:
             trans.commit()
             assert False
-        except exceptions.InvalidRequestError, e:
+        except tsa.exc.InvalidRequestError, e:
             assert str(e) == "Can't reconnect until invalid transaction is rolled back"
 
         assert trans.is_active
@@ -157,7 +159,7 @@ class MockReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError:
+        except tsa.exc.DBAPIError:
             pass
 
         assert not conn.closed
@@ -172,7 +174,7 @@ class MockReconnectTest(TestBase):
         assert not conn.invalidated
         assert len(dbapi.connections) == 1
 
-
+engine = None
 class RealReconnectTest(TestBase):
     def setUp(self):
         global engine
@@ -192,7 +194,7 @@ class RealReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError, e:
+        except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
 
@@ -208,7 +210,7 @@ class RealReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError, e:
+        except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
         assert conn.invalidated
@@ -216,7 +218,24 @@ class RealReconnectTest(TestBase):
         assert not conn.invalidated
 
         conn.close()
-    
+
+    def test_null_pool(self):
+        engine = engines.reconnecting_engine(options=dict(poolclass=pool.NullPool))
+        conn = engine.connect()
+        self.assertEquals(conn.execute(select([1])).scalar(), 1)
+        assert not conn.closed
+        engine.test_shutdown()
+        try:
+            conn.execute(select([1]))
+            assert False
+        except tsa.exc.DBAPIError, e:
+            if not e.connection_invalidated:
+                raise
+        assert not conn.closed
+        assert conn.invalidated
+        self.assertEquals(conn.execute(select([1])).scalar(), 1)
+        assert not conn.invalidated
+        
     def test_close(self):
         conn = engine.connect()
         self.assertEquals(conn.execute(select([1])).scalar(), 1)
@@ -227,7 +246,7 @@ class RealReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError, e:
+        except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
 
@@ -248,7 +267,7 @@ class RealReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.DBAPIError, e:
+        except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
 
@@ -259,7 +278,7 @@ class RealReconnectTest(TestBase):
         try:
             conn.execute(select([1]))
             assert False
-        except exceptions.InvalidRequestError, e:
+        except tsa.exc.InvalidRequestError, e:
             assert str(e) == "Can't reconnect until invalid transaction is rolled back"
 
         assert trans.is_active
@@ -267,7 +286,7 @@ class RealReconnectTest(TestBase):
         try:
             trans.commit()
             assert False
-        except exceptions.InvalidRequestError, e:
+        except tsa.exc.InvalidRequestError, e:
             assert str(e) == "Can't reconnect until invalid transaction is rolled back"
 
         assert trans.is_active
@@ -295,6 +314,7 @@ class RecycleTest(TestBase):
             self.assertEquals(conn.execute(select([1])).scalar(), 1)
             conn.close()
     
+meta, table, engine = None, None, None
 class InvalidateDuringResultTest(TestBase):
     def setUp(self):
         global meta, table, engine
@@ -307,28 +327,28 @@ class InvalidateDuringResultTest(TestBase):
         table.insert().execute(
             [{'id':i, 'name':'row %d' % i} for i in range(1, 100)]
         )
-        
+
     def tearDown(self):
         meta.drop_all()
         engine.dispose()
-    
+
+    @testing.fails_on('mysql', 'FIXME: unknown')
     def test_invalidate_on_results(self):
         conn = engine.connect()
-        
+
         result = conn.execute("select * from sometable")
         for x in xrange(20):
             result.fetchone()
-        
+
         engine.test_shutdown()
         try:
             result.fetchone()
             assert False
-        except exceptions.DBAPIError, e:
+        except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
 
         assert conn.invalidated
-    test_invalidate_on_results = testing.fails_on('mysql')    (test_invalidate_on_results)
-        
+
 if __name__ == '__main__':
     testenv.main()
