@@ -65,12 +65,20 @@ class DeclarativeTest(DeclarativeTestBase):
         self.assertRaisesMessage(sa.exc.InvalidRequestError, "does not have a __table__", go)
 
     def test_cant_add_columns(self):
-        t = Table('t', Base.metadata, Column('id', Integer, primary_key=True))
+        t = Table('t', Base.metadata, Column('id', Integer, primary_key=True), Column('data', String))
         def go():
             class User(Base):
                 __table__ = t
                 foo = Column(Integer, primary_key=True)
-        self.assertRaisesMessage(sa.exc.ArgumentError, "add additional columns", go)
+        # can't specify new columns not already in the table
+        self.assertRaisesMessage(sa.exc.ArgumentError, "Can't add additional column 'foo' when specifying __table__", go)
+
+        # regular re-mapping works tho
+        class Bar(Base):
+            __table__ = t
+            some_data = t.c.data
+            
+        assert class_mapper(Bar).get_property('some_data').columns[0] is t.c.data
     
     def test_undefer_column_name(self):
         # TODO: not sure if there was an explicit
@@ -117,7 +125,9 @@ class DeclarativeTest(DeclarativeTestBase):
             id = Column(Integer, primary_key=True)
             name = Column(String(50))
             addresses = relation("Address", order_by="desc(Address.email)", 
-                primaryjoin="User.id==Address.user_id", foreign_keys="[Address.user_id]")
+                primaryjoin="User.id==Address.user_id", foreign_keys="[Address.user_id]",
+                backref=backref('user', primaryjoin="User.id==Address.user_id", foreign_keys="[Address.user_id]")
+                )
         
         class Address(Base, ComparableEntity):
             __tablename__ = 'addresses'
@@ -766,7 +776,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         
         # compile succeeds because inherit_condition is honored
         compile_mappers()
-
+    
     def test_joined(self):
         class Company(Base, ComparableEntity):
             __tablename__ = 'companies'
@@ -835,6 +845,25 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         def go():
             assert sess.query(Person).filter(Manager.name=='dogbert').one().id
         self.assert_sql_count(testing.db, go, 1)
+
+    def test_subclass_mixin(self):
+        class Person(Base, ComparableEntity):
+            __tablename__ = 'people'
+            id = Column('id', Integer, primary_key=True)
+            name = Column('name', String(50))
+            discriminator = Column('type', String(50))
+            __mapper_args__ = {'polymorphic_on':discriminator}
+        
+        class MyMixin(object):
+            pass
+            
+        class Engineer(MyMixin, Person):
+            __tablename__ = 'engineers'
+            __mapper_args__ = {'polymorphic_identity':'engineer'}
+            id = Column('id', Integer, ForeignKey('people.id'), primary_key=True)
+            primary_language = Column('primary_language', String(50))
+            
+        assert class_mapper(Engineer).inherits is class_mapper(Person)
         
     def test_with_undefined_foreignkey(self):
         class Parent(Base):
