@@ -1,7 +1,7 @@
 import testenv; testenv.configure_for_tests()
 import datetime
 from testlib import sa, testing
-from testlib.sa import Table, Column, Integer, String, ForeignKey, MetaData
+from testlib.sa import Table, Column, Integer, String, ForeignKey, MetaData, and_
 from testlib.sa.orm import mapper, relation, backref, create_session, compile_mappers, clear_mappers
 from testlib.testing import eq_, startswith_
 from orm import _base, _fixtures
@@ -650,6 +650,79 @@ class RelationTest6(_base.MappedTest):
             [TagInstance(data='iplc_case'), TagInstance(data='not_iplc_case')]
         )
 
+class AmbiguousJoinInterpretedAsSelfRef(_base.MappedTest):
+    """test ambiguous joins due to FKs on both sides treated as self-referential.
+    
+    this mapping is very similar to that of test/orm/inheritance/query.py
+    SelfReferentialTestJoinedToBase , except that inheritance is not used
+    here.
+    
+    """
+    
+    def define_tables(self, metadata):
+        subscriber_table = Table('subscriber', metadata,
+           Column('id', Integer, primary_key=True),
+           Column('dummy', String(10)) # to appease older sqlite version
+          )
+
+        address_table = Table('address',
+                 metadata,
+                 Column('subscriber_id', Integer, ForeignKey('subscriber.id'), primary_key=True),
+                 Column('type', String(1), primary_key=True),
+                 )
+
+    @testing.resolve_artifact_names
+    def setup_mappers(self):
+        subscriber_and_address = subscriber.join(address, 
+        	and_(address.c.subscriber_id==subscriber.c.id, address.c.type.in_(['A', 'B', 'C'])))
+
+        class Address(_base.ComparableEntity):
+            pass
+
+        class Subscriber(_base.ComparableEntity):
+            pass
+
+        mapper(Address, address)
+
+        mapper(Subscriber, subscriber_and_address, properties={
+           'id':[subscriber.c.id, address.c.subscriber_id],
+           'addresses' : relation(Address, 
+                backref=backref("customer"))
+           })
+        
+    @testing.resolve_artifact_names
+    def test_mapping(self):
+        from sqlalchemy.orm.interfaces import ONETOMANY, MANYTOONE
+        sess = create_session()
+        assert Subscriber.addresses.property.direction is ONETOMANY
+        assert Address.customer.property.direction is MANYTOONE
+        
+        s1 = Subscriber(type='A',
+                addresses = [
+                    Address(type='D'),
+                    Address(type='E'),
+                ]
+        )
+        a1 = Address(type='B', customer=Subscriber(type='C'))
+        
+        assert s1.addresses[0].customer is s1
+        assert a1.customer.addresses[0] is a1
+        
+        sess.add_all([s1, a1])
+        
+        sess.flush()
+        sess.expunge_all()
+        
+        eq_(
+            sess.query(Subscriber).order_by(Subscriber.type).all(),
+            [
+                Subscriber(id=1, type=u'A'), 
+                Subscriber(id=2, type=u'B'), 
+                Subscriber(id=2, type=u'C')
+            ]
+        )
+
+
 class ManualBackrefTest(_fixtures.FixtureTest):
     """Test explicit relations that are backrefs to each other."""
 
@@ -1446,7 +1519,64 @@ class ExplicitLocalRemoteTest(_base.MappedTest):
         mapper(T2, t2)
         self.assertRaises(sa.exc.ArgumentError, sa.orm.compile_mappers)
 
+class InvalidRemoteSideTest(_base.MappedTest):
+    def define_tables(self, metadata):
+        Table('t1', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(50)),
+            Column('t_id', Integer, ForeignKey('t1.id'))
+            )
 
+    @testing.resolve_artifact_names
+    def setup_classes(self):
+        class T1(_base.ComparableEntity):
+            pass
+
+    @testing.resolve_artifact_names
+    def test_o2m_backref(self):
+        mapper(T1, t1, properties={
+            't1s':relation(T1, backref='parent')
+        })
+
+        self.assertRaisesMessage(sa.exc.ArgumentError, "T1.t1s and back-reference T1.parent are "
+                    "both of the same direction <symbol 'ONETOMANY>.  Did you "
+                    "mean to set remote_side on the many-to-one side ?", sa.orm.compile_mappers)
+
+    @testing.resolve_artifact_names
+    def test_m2o_backref(self):
+        mapper(T1, t1, properties={
+            't1s':relation(T1, backref=backref('parent', remote_side=t1.c.id), remote_side=t1.c.id)
+        })
+
+        self.assertRaisesMessage(sa.exc.ArgumentError, "T1.t1s and back-reference T1.parent are "
+                    "both of the same direction <symbol 'MANYTOONE>.  Did you "
+                    "mean to set remote_side on the many-to-one side ?", sa.orm.compile_mappers)
+
+    @testing.resolve_artifact_names
+    def test_o2m_explicit(self):
+        mapper(T1, t1, properties={
+            't1s':relation(T1, back_populates='parent'),
+            'parent':relation(T1, back_populates='t1s'),
+        })
+
+        # can't be sure of ordering here
+        self.assertRaisesMessage(sa.exc.ArgumentError, 
+                    "both of the same direction <symbol 'ONETOMANY>.  Did you "
+                    "mean to set remote_side on the many-to-one side ?", sa.orm.compile_mappers)
+
+    @testing.resolve_artifact_names
+    def test_m2o_explicit(self):
+        mapper(T1, t1, properties={
+            't1s':relation(T1, back_populates='parent', remote_side=t1.c.id),
+            'parent':relation(T1, back_populates='t1s', remote_side=t1.c.id)
+        })
+
+        # can't be sure of ordering here
+        self.assertRaisesMessage(sa.exc.ArgumentError, 
+                    "both of the same direction <symbol 'MANYTOONE>.  Did you "
+                    "mean to set remote_side on the many-to-one side ?", sa.orm.compile_mappers)
+
+        
 class InvalidRelationEscalationTest(_base.MappedTest):
 
     def define_tables(self, metadata):
